package br.usp.ime.mac0331.sgc;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Shape;
import java.awt.Stroke;


public class GeometriaShape implements Comparable<GeometriaShape> {
	private DrawScheme scheme;

	private Shape shape;

	public GeometriaShape(Shape shape, DrawScheme scheme) {
		this.shape = shape;
		this.scheme = scheme;
	}

	public DrawScheme getScheme() {
		return scheme;
	}

	public Shape getShape() {
		return shape;
	}

	public int compareTo(GeometriaShape o) {
		GeometriaShape other = (GeometriaShape) o;

		return GeometriaShape.DrawScheme.getPriority(other.getScheme())
				.getValue()
				- GeometriaShape.DrawScheme.getPriority(this.scheme).getValue();
	}

	public Color getColor() {
		return DrawScheme.getColor(this.scheme);
	}

	public Stroke getStroke() {
		return DrawScheme.getStroke(this.scheme, 0);
	}

	public enum DrawScheme {
		SIMPLE_POLYGON, TRIANGLE, MOVING_TRIANGLE, POINT, LINE;

		public static Color getColor(DrawScheme scheme) {

			switch (scheme) {
			case SIMPLE_POLYGON:
				return Color.black;
			case TRIANGLE:
				return Color.blue;
			case MOVING_TRIANGLE:
				return Color.green;
			case POINT:
				return Color.red;
			case LINE:
				return Color.black;
			}

			return Color.BLACK;
		}

		public static boolean mustFill(DrawScheme scheme) {

			switch (scheme) {
			case POINT:
				return true;
			}
			return false;
		}

		public static Stroke getStroke(DrawScheme scheme, float d) {

			switch (scheme) {
			case SIMPLE_POLYGON:
				return new BasicStroke(4f + d);
			case TRIANGLE:
				return new BasicStroke(4f + d);
			case MOVING_TRIANGLE:
				return new BasicStroke(4f + d);
			case LINE:
				return new BasicStroke(4f + d);
			}

			return new BasicStroke(4f + d);
		}

		public static Priority getPriority(DrawScheme scheme) {

			switch (scheme) {
			case POINT:
				return Priority.HIGHEST;
			}

			return Priority.NORMAL;
		}
	}
}
