package br.usp.ime.mac0331.sgc;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JOptionPane;

import org.apache.batik.svggen.SVGGraphics2D;
import org.apache.batik.svggen.SVGGraphics2DIOException;

import br.usp.ime.mac0331.sgc.drawable.Figure;

public class Simulation {
	private List<String> steps;

	private Map<String, Integer> mainStepsMap;

	private int indiceCurrentStep;

	private Step currentStep;

	public Simulation() {
		this.mainStepsMap = new HashMap<String, Integer>();
		this.steps = new ArrayList<String>();
		this.indiceCurrentStep = -1;
	}

	public void addStep(String step) {
		this.steps.add(step);
	}

	public void next(int steps) {
		if (this.indiceCurrentStep + steps < this.steps.size()) {
			this.indiceCurrentStep += steps;
		}
	}

	public void previous(int steps) {
		if (this.indiceCurrentStep - steps >= 0) {
			this.indiceCurrentStep = this.indiceCurrentStep - steps;
		}
	}

	public BufferedImage getBufferedImageActual() {
		BufferedImage buffer = new BufferedImage(800, 800,
				BufferedImage.TYPE_INT_ARGB_PRE);
		Graphics2D graphics2D = buffer.createGraphics();

		drawActualShapes(graphics2D);

		return buffer;
	}

	public void takeSnapshot() {
		this.takeSnapshot(this.indiceCurrentStep, true);
	}

	public void takeSnapshot(int x, boolean message) {
		File file;
		try {
			SVGGraphics2D g = new SVGSnapshot().createGraphics();
			this.drawActualShapes(g);

			file = new File("" + x + ".svg");
			Writer out = new OutputStreamWriter(new FileOutputStream(file),
					"UTF-8");
			g.stream(out, true);

		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			JOptionPane.showConfirmDialog(null, "Cannot save file: " + e);
			return;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			JOptionPane.showConfirmDialog(null, "Cannot save file: " + e);
			return;
		} catch (SVGGraphics2DIOException e) {
			e.printStackTrace();
			JOptionPane.showConfirmDialog(null, "Cannot save file: " + e);
			return;
		}
		if (message) {
			JOptionPane.showMessageDialog(null, "SVG saved: "
					+ file.getAbsolutePath());
		}
	}

	public int getStep() {
		return this.indiceCurrentStep;
	}

	public int getStepCount() {
		return this.steps.size();
	}

	public void goTo(String step) {
		this.indiceCurrentStep = this.mainStepsMap.get(step);
	}

	public void setStep(String step) {
		this.mainStepsMap.put(step, this.steps.size() - 1);
		// takeSnapshot(this.steps.size() - 1, false);
	}

	private void drawActualShapes(Graphics2D graphics2D) {
		graphics2D.translate(20, 20);
		graphics2D.setStroke(new BasicStroke(1.0f));

		this.currentStep = new Step(this.steps.get(this.indiceCurrentStep));

		List<GeometriaShape> shapes = new ArrayList<GeometriaShape>();
		for (Figure figure : this.currentStep.getBackGround()) {
			shapes.addAll(figure.getShapes());
		}
		Collections.sort(shapes);
		draw(graphics2D, shapes, -.5f);

		shapes.clear();
		for (Figure figure : this.currentStep.getForeground()) {
			shapes.addAll(figure.getShapes());
		}
		Collections.sort(shapes);
		draw(graphics2D, shapes, 0);
	}

	private void draw(Graphics2D g, List<GeometriaShape> shapes, float delta) {

		for (GeometriaShape myShape : shapes) {
			g.setColor(GeometriaShape.DrawScheme.getColor(myShape.getScheme()));
			g.setStroke(GeometriaShape.DrawScheme.getStroke(
					myShape.getScheme(), delta));

			if (GeometriaShape.DrawScheme.mustFill(myShape.getScheme())) {
				g.fill(myShape.getShape());
			} else {
				g.draw(myShape.getShape());
			}
		}
	}

	public Map<String, Integer> getMainStepsMap() {
		return new HashMap<String, Integer>(this.mainStepsMap);
	}

	public String getStatistics() {
		if (this.currentStep == null) {
			return "";
		}
		
		StringBuilder builder = new StringBuilder();
		builder.append("Estatsticas \n");
	
		for (String key : this.currentStep.getStatistics().keySet()) {
			String value = this.currentStep.getStatistics().get(key);
			builder.append(key + " = " + value + "\n");
		}
		return builder.toString();
	}
}
