package br.usp.ime.mac0331.sgc;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextArea;

public class Simulator implements ActionListener {

	private JButton next = new JButton(">");

	private JButton previous = new JButton("<");

	private JButton fastForward = new JButton(">>");

	private JButton rewind = new JButton("<<");

	private JButton run = new JButton("run");
	
	private JTextArea statistcs = new JTextArea();

	private JButton screenshot = new JButton("screenshot");

	private JProgressBar progressBar;

	private JFrame frame;

	private JPanel controls = new JPanel();

	private JPanel main = new JPanel();

	private Simulation simulation;

	public Simulator(Simulation simulation) {
		this.simulation = simulation;
		this.frame = new JFrame("Geometria Computacional");
		this.frame.setLayout(new BorderLayout());

		this.progressBar = new JProgressBar(0, this.simulation.getStepCount() - 1);
		
		this.controls.setLayout(new FlowLayout());
		this.controls.add(this.rewind);
		this.controls.add(this.previous);
		this.controls.add(this.progressBar);
		this.controls.add(this.next);
		this.controls.add(this.fastForward);

		for (String s : this.simulation.getMainStepsMap().keySet()) {
			JButton button = new JButton(s);
			button.setEnabled(false);
			this.controls.add(button);
			button.addActionListener(this);
			button.setActionCommand(s);
		}

		this.controls.add(run);
		this.controls.add(screenshot);

		this.rewind.addActionListener(this);
		this.previous.addActionListener(this);
		this.next.addActionListener(this);
		this.fastForward.addActionListener(this);
		this.run.addActionListener(this);
		this.screenshot.addActionListener(this);

		this.next.setEnabled(false);
		this.previous.setEnabled(false);
		this.fastForward.setEnabled(false);
		this.rewind.setEnabled(false);
		this.screenshot.setEnabled(false);

		this.open();
	}

	public void execute() {
		for (Component component : this.controls.getComponents()) {
			if (component instanceof JButton) {
				JButton button = (JButton) component;
				button.setEnabled(true);
			}
		}
		this.run.setEnabled(false);
		this.progressBar.setMaximum(this.simulation.getStepCount());
	}

	public void open() {

		this.frame.add(this.controls, BorderLayout.NORTH);
		this.controls.setBackground(Color.WHITE);
		this.frame.add(this.main, BorderLayout.CENTER);
		this.main.setBackground(Color.WHITE);
		this.frame.add(this.statistcs, BorderLayout.EAST);
		this.statistcs.setBackground(Color.WHITE);

		this.frame.setSize(1000, 700);
		this.frame.setResizable(false);
		this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.frame.setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		JButton button = (JButton) e.getSource();

		if (button == this.next) {
			this.simulation.next(1);
		} else if (button == this.previous) {
			this.simulation.previous(1);
		} else if (button == this.fastForward) {
			this.simulation.next(10);
		} else if (button == this.rewind) {
			this.simulation.previous(10);
		} else if (button == this.screenshot) {
			this.simulation.takeSnapshot();
		} else if (button == this.run) {
			this.execute();
			return;
		} else {
			String step = button.getActionCommand();
			this.simulation.goTo(step);
		}	
		
		if (this.main.getComponentCount() > 0)
			this.main.remove(0);
		this.main.add(new JLabel(new ImageIcon(this.simulation
				.getBufferedImageActual())));

		this.progressBar.setValue(this.simulation.getStep() + 1);
		this.statistcs.setText(this.simulation.getStatistics());
		
		this.controls.repaint();
		this.frame.setVisible(true);
	}

	public Simulation getSimulation() {
		return this.simulation;
	}

}
