package br.usp.ime.mac0331.sgc;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.apache.log4j.Logger;

import br.usp.ime.mac0331.sgc.drawable.Drawable;
import br.usp.ime.mac0331.sgc.drawable.DrawableFactory;
import br.usp.ime.mac0331.sgc.drawable.Figure;

public class Step {
	
	private static Logger logger = Logger.getLogger(Step.class);

	private List<Figure> backGround = new ArrayList<Figure>();

	private List<Figure> foreground = new ArrayList<Figure>();

	private Map<String, String> statistics = new HashMap<String, String>();

	private String name;

	public Step(String step) {
		
		this.stringToStep(step);
		
	}

	private void stringToStep(String step) {
		Step.logger.debug("Processando o step: \n" + step);
		Scanner input = new Scanner(step);
		input.useDelimiter("step_");

		while (input.hasNext()) {
			String item = input.next();
			Step.logger.debug("Processando item: \n" + item);
			if (item.startsWith("name")) {
				this.name = item.substring(4).trim();
				Step.logger.debug("Nome do step adicionado: " + this.name);
			} else if (item.startsWith("bg")) {
				this.stringToBackground(item.substring(3).trim());
			} else if (item.startsWith("fg")) {
				this.stringToForenground(item.substring(3).trim());
			} else if (item.startsWith("stat")) {
				this.stringToStatistics(item.substring(4).trim());
			}
		}
	}

	private void stringToStatistics(String statistics) {
		Step.logger.debug("Processando estatsticas: \n" + statistics);
		Scanner input = new Scanner(statistics);
		input.useDelimiter("var");

		while (input.hasNext()) {
			String var = input.next().trim();
			Step.logger.debug("Processando varivel: \n" + var);
			Scanner inputVar = new Scanner(var);
			this.statistics.put(inputVar.next(), inputVar.next());
		}
	}

	private void stringToForenground(String forenground) {
		Step.logger.debug("Processando forenground: \n" + forenground);
		Scanner input = new Scanner(forenground);
		input.useDelimiter("fig_");
		
		while (input.hasNext()) {
			Drawable drawable = DrawableFactory.createDrawable(input.next().trim());
			this.addFiguresInForeground(drawable);
		}
	}

	private void stringToBackground(String background) {
		Step.logger.debug("Processando background: \n" + background);
		Scanner input = new Scanner(background);
		input.useDelimiter("fig_");
		
		while (input.hasNext()) {
			Drawable drawable = DrawableFactory.createDrawable(input.next().trim());
			this.addFiguresInBackground(drawable);
		}

	}

	public void addFiguresInForeground(Drawable... list) {
		this.addFiguresInForeground(Arrays.asList(list));
	}

	public void addFiguresInForeground(List<Drawable> drawables) {
		for (Drawable drawable : drawables) {
			this.foreground.add(drawable.toFigure());
		}
	}

	public void addFiguresInBackground(List<? extends Drawable> drawables) {

		for (Drawable drawable : drawables) {
			this.backGround.add(drawable.toFigure());
		}
	}

	public void addFiguresInBackground(Drawable... list) {
		this.addFiguresInBackground(Arrays.asList(list));
	}

	public List<Figure> getBackGround() {
		return this.backGround;
	}

	public List<Figure> getForeground() {
		return this.foreground;
	}

	public Map<String, String> getStatistics() {
		return new HashMap<String, String>(this.statistics);
	}
	
	public String getName() {
		return this.name;
	}
}
