package br.usp.ime.mac0331.sgc.drawable;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import org.apache.log4j.Logger;

public class DrawableFactory {
	private static Logger logger = Logger.getLogger(DrawableFactory.class);

	public static Drawable createDrawable(String drawable) {
		DrawableFactory.logger.debug("Criando o drawable\n" + drawable);
		if (drawable.startsWith("pt")) {
			return DrawableFactory.createPoint(drawable.substring(2).trim());
		} else if (drawable.startsWith("ln")) {
			return DrawableFactory.createLine(drawable.substring(2).trim());
		} else if (drawable.startsWith("pg")) {
			return DrawableFactory.createPolygon(drawable.substring(2).trim());
		}
		throw new IllegalArgumentException(
				"As figuras suportadas so: pontos(pt), linha(ln), polgono(pg)");
	}

	private static Drawable createPolygon(String polygon) {
		DrawableFactory.logger.debug("Criando um polgono\n" + polygon);
		Scanner input = new Scanner(polygon);
		List<Point> points = new ArrayList<Point>();

		while (input.hasNextDouble()) {
			double x = input.nextDouble();
			double y = input.nextDouble();
			points.add(new Point(x, y));
		}

		if (points.size() < 3) {
			throw new IllegalArgumentException(
					"O polgono deve ter no mnimo 3 pontos");
		}
		return new Polygon(points);

	}

	private static Drawable createLine(String line) {
		DrawableFactory.logger.debug("Criando uma linha\n" + line);
		Scanner input = new Scanner(line);
		double x1 = input.nextDouble();
		double y1 = input.nextDouble();
		double x2 = input.nextDouble();
		double y2 = input.nextDouble();
		return new Line(new Point(x1, y1), new Point(x2, y2));

	}

	private static Drawable createPoint(String point) {
		DrawableFactory.logger.debug("Criando um ponto\n" + point);
		Scanner input = new Scanner(point);
		double x = input.nextDouble();
		double y = input.nextDouble();
		return new Point(x, y);
	}
}
